<?php
/***** BEGIN MIT LICENSE BLOCK *****
 *
 * Copyright (C) 2012 Esponce d.o.o.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 ***** END MIT LICENCE BLOCK *****/
 
/**************************************************************************************//**
 * Replaces shortcodes with cached images where possible.
 ******************************************************************************************/
function esponce_cache_content_filter($content)
{
	$matches = null;
	preg_match_all('/\[qrcode(.*?)\]/i', $content, $matches);
	
	foreach ($matches[0] as $count => $shortcode)
	{
		//Get parameters
		$p = esponce_get_shortcode_parameters($shortcode);
		
		//Retrieve cached files
		$filename = esponce_cache_get_filename($shortcode);
		if (esponce_cache_image_exists($filename))
		{
			//Build URL and "img" element
			$url = esponce_cache_get_url($filename);
			$html = esponce_get_shortcode_html($url, $p["alt"], $p["tooltip"], $p["class"], $p["style"], $p["width"], $p["height"]);
			
			//Replace $shortcode with $html
			$content = str_replace($shortcode, $html, $content);
		}
	}
	
	return $content;
}

/**************************************************************************************//**
 * Saves image to local storage to cache it.
 ******************************************************************************************/
function esponce_cache_save_image($filename, $url)
{
	//Both parameters must be set
	if (empty($filename) || empty($url))
	{
		//Otherwise skip caching
		return;
	}

	//Get the upload directory
	$upload_dir = wp_upload_dir();
	$path = trailingslashit($upload_dir['path']) . $filename;
	
	//Download image and save it to file
	$content = file_get_contents($url);
	file_put_contents($path, $content);
	
	//global $wp_filesystem;
	//$wp_filesystem->put_contents($filename, 'Test file contents', FS_CHMOD_FILE));
}

/**************************************************************************************//**
 * Checks if cached image exists.
 ******************************************************************************************/
function esponce_cache_image_exists($filename)
{
	//Parameter must be set
	if (empty($filename))
	{
		return false;
	}

	//Get the upload directory
	$upload_dir = wp_upload_dir();
	$path = trailingslashit($upload_dir['path']) . $filename;
	
	$result = file_exists($path);
	return $result;
}

/**************************************************************************************//**
 * Gets cached image URL.
 ******************************************************************************************/
function esponce_cache_get_url($filename)
{
	//Parameter must be set
	if (empty($filename))
	{
		return null;
	}
	
	$upload_dir = wp_upload_dir();
	$url = $upload_dir['url'] . "/" . $filename;
	return $url;
}

/**************************************************************************************//**
 * Builds a file name from shortcode.
 ******************************************************************************************/
function esponce_cache_get_filename($shortcode)
{
	return "esp-qrcode-" . md5($shortcode) . ".png";
}

/**************************************************************************************//**
 * Saves widget image to local storage to cache it.
 ******************************************************************************************/
function esponce_cache_widget_qrcode($id, $url)
{
	//Both parameters must be set
	if (empty($id) || empty($url))
	{
		//Otherwise skip caching
		return null;
	}
	
	$filename = "esp-qrcode-widget-" . $id . ".png";

	try
	{
		//Get the upload directory
		$upload_dir = wp_upload_dir();
		$upload_path = trailingslashit($upload_dir['basedir']) . "/" . $filename;
		$upload_url = $upload_dir['baseurl'] . "/" . $filename;
		
		//Log base dir and url
		//esponce_log($upload_dir['basedir']); //e.g. D:\htdocs\wordpress/wp-content/uploads
		//esponce_log($upload_dir['baseurl']); //e.g. http://localhost/wordpress/wp-content/uploads
		
		//Download image and save it to file
		$content = file_get_contents($url);
		if (empty($content))
		{
			throw new Exception("Content cannot be empty!");
		}
		
		file_put_contents($upload_path, $content);
		
		//global $wp_filesystem;
		//$wp_filesystem->put_contents($filename, 'Test file contents', FS_CHMOD_FILE));
		
		return $upload_url;	
	}
	catch (Exception $ex)
	{
		//No permissions to save file?
		//esponce_log($ex->getMessage());
	}
	
	return null;
}
?>