<?php
/***** BEGIN MIT LICENSE BLOCK *****
 *
 * Copyright (C) 2012 Esponce d.o.o.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 ***** END MIT LICENCE BLOCK *****/
 
$esponce_log_enabled = true; //$GLOBALS['post']->post_name == 'debug'

/**************************************************************************************//**
 * Logs content to a file.
 ******************************************************************************************/
function esponce_log($content)
{
	//file_put_contents("d:\\Shared\\log.txt", $content . "\n", FILE_APPEND);
	//return;

	global $esponce_log_enabled;
	if ($esponce_log_enabled)
	{
		$upload_dir = wp_upload_dir();
		$path = trailingslashit($upload_dir['path']) . 'esponce-qrcode.log';
		file_put_contents($path, $content . "\n", FILE_APPEND);
	}
}

/**************************************************************************************//**
 * Builds "img" element with attributes.
 ******************************************************************************************/
function esponce_get_shortcode_html($url, $alt = null, $tooltip = null, $class = null, $style = null, $width = null, $height = null)
{
	$html = "<!-- QR Code generated by Esponce: http://www.esponce.com/ -->";
	$html .= "<img src=\"$url\"";
	$html .= " alt=\"" . (!empty($alt) ? htmlspecialchars($alt) : "") . "\"";
	$html .= (!empty($tooltip) ? " title=\"" . htmlspecialchars($tooltip) . "\"" : "");
	$html .= " class=\"esponce-qrcode" . (!empty($class) ? " $class" : "") . "\"";
	$html .= (!empty($style) ? " style=\"$style\"" : "");
	$html .= (!empty($width) ? " width=\"$width\"" : "");
	$html .= (!empty($height) ? " height=\"$height\"" : "");
	$html .= " />";
	return $html;
}

/**************************************************************************************//**
 * Encodes shortcode component value.
 ******************************************************************************************/
function esponce_encode_shortcode_value($value)
{
	//Value must be set to continue
	if (empty($value)) return $value;
	
	$result = $value;
	$result = str_replace('"', "&quot;", $result); //Double quotes
	$result = str_replace("\r\n", "&br;", $result); //New lines
	$result = str_replace("\r", "&br;", $result); //New lines
	$result = str_replace("\n", "&br;", $result); //New lines
	return $result;
}

/**************************************************************************************//**
 * Decodes shortcode component value.
 ******************************************************************************************/
function esponce_decode_shortcode_value($value)
{
	//Value must be set to continue
	if (empty($value)) return $value;
	
	$result = $value;
	$result = str_replace("&quot;", '"', $result); //Double quotes
	$result = str_replace("&br;", "\n", $result); //New lines
	return $result;
}

/**************************************************************************************//**
 * Parses a shortcode to get parameters.
 ******************************************************************************************/
function esponce_get_shortcode_parameters($shortcode)
{
	$result = array
	(
		"content" => esponce_get_parameter($shortcode, "content"),
		"caption" => esponce_get_parameter($shortcode, "caption"),
		"tooltip" => esponce_get_parameter($shortcode, "tooltip"),
		"alt" => esponce_get_parameter($shortcode, "alt"),
		"class" => esponce_get_parameter($shortcode, "class"),
		"style" => esponce_get_parameter($shortcode, "style"),
		"width" => esponce_get_parameter($shortcode, "width"),
		"height" => esponce_get_parameter($shortcode, "height"),
		"foreground" => esponce_get_parameter($shortcode, "foreground"),
		"background" => esponce_get_parameter($shortcode, "background"),
		"version" => esponce_get_parameter($shortcode, "version"),
		"padding" => esponce_get_parameter($shortcode, "padding"),
		"size" => esponce_get_parameter($shortcode, "size"),
		"ec" => esponce_get_parameter($shortcode, "ec"),
		"em" => esponce_get_parameter($shortcode, "em")
	);
	return $result;
}

/**************************************************************************************//**
 * Gets a parameter value from shortcode.
 ******************************************************************************************/
function esponce_get_parameter($shortcode, $name)
{
	$matches = null;
	$pattern = '/' . $name . '="(.*?)"/i';
	if (preg_match($pattern, $shortcode, $matches))
	{
		$result = esponce_decode_shortcode_value($matches[1]);
		return $result;
	}
	return null;
}

/**************************************************************************************//**
 * Builds an URL from a shortcode.
 ******************************************************************************************/
function esponce_get_url_from_shortcode($shortcode)
{	
	$p = esponce_get_shortcode_parameters($shortcode);
	
	//Set variables and skip default values
	$content = $p["content"]; //Content
	$format = "png"; //Format
	$size = (!empty($p["size"]) && @intval($p["size"]) > 0 ? $p["size"] : null); //Size
	$version = (!empty($p["version"]) && @intval($p["version"]) > 0 ? $p["version"] : null); //Version
	$padding = (!empty($p["padding"]) && @intval($p["padding"]) >= 0 ? $p["padding"] : null); //Padding
	$foreground = (!empty($p["foreground"]) && $p["foreground"] != "#000000" ? $p["foreground"] : null); //Foreground
	$background = (!empty($p["background"]) && $p["background"] != "#FFFFFF" ? $p["background"] : null); //Background
	$em = (!empty($p["em"]) && $p["em"] != "byte" ? $p["em"] : null); //Encode mode
	$ec = (!empty($p["ec"]) && $p["ec"] != "M" ? $p["ec"] : null); //Error correction
	
	//Build URL to generate a QR Code
	$api = new EsponceQRCodeAPI();
	$url = $api->getUrl($content, $format, $version, $size, $padding, $em, $ec, $foreground, $background);
	esponce_log("$shortcode -> $url");
	
	return $url;
}

?>