<?php
/***** BEGIN MIT LICENSE BLOCK *****
 *
 * Copyright (C) 2012 Esponce d.o.o.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 ***** END MIT LICENCE BLOCK *****/
 
/**************************************************************************************//**
 * Widget
 ******************************************************************************************/
class EsponceQRCodeWidget extends WP_Widget
{
	function EsponceQRCodeWidget()
	{
		$args = array
		(
			"description" => __("Use this widget to display a QR Code on sidebar.")
		);
		$this->WP_Widget("EsponceQRCodeWidget", "QR Code by Esponce", $args); //Base ID, Name, Args
	}
	
	/**************************************************************************************//**
	 * Gets a permalink - URL of the page where QR Code is located
	 ******************************************************************************************/
	function getPermalink()
	{
		//Find permalink - may be too complex to handle?
		//if (is_single())
		//{
		//	$post = get_post($post_id); 
		//	$content = get_permalink($post->ID);
		//}
		//else if (is_category())
		//{
		//	//$content = get_category_link($id); //TODO: Get category id
		//}
		
		//$content = $_SERVER['PATH_INFO']; //Current page URL?
		
		//Get current URL, it is more simple approach and reliable
		$result = ($_SERVER["HTTPS"] == "on" ? "https" : "http") . "://";
		$result .= $_SERVER["SERVER_NAME"];
		if ($_SERVER["SERVER_PORT"] != "80")
		{
			$result .= ":" . $_SERVER["SERVER_PORT"];
		}
		
		if (ESPONCE_PERMALINK_WITH_QUERY)
		{
			//If user prefers full URL with query strings
			$result .= $_SERVER["REQUEST_URI"];
		}
		else
		{
			//If user prefers URL without query strings, smaller QR Code
			//$result .= $_SERVER["SCRIPT_NAME"]; //This will display script name, e.g. "/index.php", not good if on rewrite
			
			$uri = $_SERVER["REQUEST_URI"];
			$pos = strpos($uri, "?");
			if ($pos !== false)
			{
				$uri = substr($uri, 0, $pos);
			}
			$result .= $uri;
		}
		//Another possibility is to create full link and shorten it using URL shortener service, e.g. goo.by, bit.ly
		
		return $result;
	}

	/**************************************************************************************//**
	 * Renders a widget
	 * @see WP_Widget::widget()
	 * @param array $args     Widget arguments.
	 * @param array $instance Saved values from database.
	 ******************************************************************************************/
	function widget($args, $instance)
	{
		extract($args);
		$cache = $instance['cache'];
		$wid = $instance['wid']; //Widget id
		$title = apply_filters('widget_title', $instance['title']);
		$permalink = ($instance['permalink'] == "1");
		$content = $instance['content'];
		$caption = $instance['caption'];
		$tooltip = $instance['tooltip'];
		$alt = $instance['alt'];
		
		if ($permalink)
		{
			//Get current page URL
			$content = $this->getPermalink();
		}
		
		$url = "";
		if (!$permalink && !empty($cache) && ESPONCE_ENABLE_CACHING)
		{
			//Use cached image
			$url = $cache;
		}
		else
		{
			//Get parameters
			$module_color = $instance['module_color'];
			$background_color = $instance['background_color'];
			$module_size = str_replace(" ", "", $instance['module_size']); //Trim spaces
			$padding = str_replace(" ", "", $instance['padding']); //Trim spaces
			
			$api = new EsponceQRCodeAPI();
			$url = $api->getURL($content, "png", null, $module_size, $padding, null, null, $module_color, $background_color);
		}
		
		//Widget title
		echo $before_widget;
		if (!empty($title))
		{
			echo $before_title . $title . $after_title;
		}
		
		//Image alignment inside container
		$style = "";
		switch ($instance['align'])
		{
			//case "1": $style = "text-align: left;"; break; //Commented to leave empty, for backward compatibility
			case "2": $style = "text-align: center;"; break;
			case "3": $style = "text-align: right;"; break;
		}
		
		//Optional tooltip on mouse over
		$title_attr = (!empty($tooltip) ? "title=\"" . htmlspecialchars($tooltip) . "\"" : "");
		
		//Image container, required to align
		echo "<div class=\"esponce-qrcode-container\" style=\"$style\">";
		
		//Actual QR Code image
		echo "<!-- QR Code generated by Esponce: http://www.esponce.com/ -->";
		echo "<img id=\"$wid\" src=\"$url\" alt=\"" . (!empty($alt) ? htmlspecialchars($alt) : "") . "\" $title_attr class=\"esponce-qrcode-widget\" />";
		
		echo "</div>";
		
		//Optional text below image
		if (!empty($caption))
		{
			echo "<div class=\"esponce-qrcode-caption\" style=\"$style\">$caption</div>";
		}
		
		echo $after_widget;
	}
	
	/**************************************************************************************//**
	 * Saves form values
	 * @see WP_Widget::update()
	 * @param array $new_instance Values just sent to be saved.
	 * @param array $old_instance Previously saved values from database.
	 * @return array Updated safe values to be saved.
	 ******************************************************************************************/
	function update($new_instance, $old_instance)
	{
		$instance = $old_instance;
		$instance['wid'] = $new_instance['wid'];
		$instance['title'] = strip_tags($new_instance['title']);
		$instance['content'] = $new_instance['content'];
		$instance['caption'] = $new_instance['caption'];
		$instance['tooltip'] = $new_instance['tooltip'];
		$instance['module_color'] = $new_instance['module_color'];
		$instance['background_color'] = $new_instance['background_color'];
		$instance['module_size'] = $new_instance['module_size'];
		$instance['padding'] = $new_instance['padding'];
		$instance['align'] = $new_instance['align'];
		$instance['alt'] = $new_instance['alt'];
		$instance['permalink'] = !empty($new_instance['permalink']) ? "1" : "0";
		$instance['advanced'] = !empty($new_instance['advanced']) ? "1" : "0";
		
		$api = new EsponceQRCodeAPI();
		$url = $api->getURL($instance['content'], "png", null, $instance['module_size'], $instance['padding'], null, null, $instance['module_color'], $instance['background_color']);
		
		$id = $instance['wid'];
		if (ESPONCE_ENABLE_CACHING)
		{
			$instance['cache'] = esponce_cache_widget_qrcode($id, $url);
		}
		
		return $instance;
	}
	
	/**************************************************************************************//**
	 * Renders widget editor in CMS.
	 * @see WP_Widget::form()
	 * @param array $instance Previously saved values from database.
	 ******************************************************************************************/
	function form($instance)
	{
		$update = ($instance ? true : false);
		$wid = ($instance ? esc_attr($instance['wid']) : __("" . uniqid(), "text_domain"));
		$title = ($instance ? esc_attr($instance['title']) : __("QR Code", "text_domain"));
		$content = ($instance ? esc_attr($instance['content']) : __("http://" . $_SERVER["HTTP_HOST"], "text_domain"));
		$caption = ($instance ? esc_attr($instance['caption']) : __("", "text_domain"));
		$tooltip = ($instance ? esc_attr($instance['tooltip']) : __("", "text_domain"));
		$module_color = ($instance ? esc_attr($instance['module_color']) : __("black", "text_domain"));
		$background_color = ($instance ? esc_attr($instance['background_color']) : __("transparent", "text_domain"));
		$module_size = ($instance ? esc_attr($instance['module_size']) : __("3", "text_domain"));
		$padding = ($instance ? esc_attr($instance['padding']) : __("0", "text_domain"));
		$align = ($instance ? esc_attr($instance['align']) : __("1", "text_domain"));
		$alt = ($instance ? esc_attr($instance['alt']) : __("", "text_domain"));
		$permalink = isset($instance['permalink']) ? ($instance['permalink'] == "1") : true;
		$advanced = isset($instance['advanced']) ? ($instance['advanced'] == "1") : false;
		
		//JavaScript name, safe for methond naming
		$safeid = str_replace("-", "_", $wid);
		
		?>
		<style type="text/css">
		.advanced_<?php echo $safeid; ?> input, .advanced_<?php echo $safeid; ?> select
		{
			background: #FFFBCC;
		}
		</style>
		
		<!-- Widget id -->
		<input type="hidden" id="<?php echo $this->get_field_id( 'wid' ); ?>" name="<?php echo $this->get_field_name( 'wid' ); ?>" value="<?php echo $wid; ?>" />
		
		<!-- Widget title -->
		<p>
			<label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e( 'Title:' ); ?></label> 
			<input class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" type="text" value="<?php echo $title; ?>" />
		</p>
		
		<!-- Content for QR Code -->
		<p>
			<label for="<?php echo $this->get_field_id( 'content' ); ?>" title="Enter hyperlink or message"><?php _e( 'Content:' ); ?></label> 
			<input class="widefat" id="<?php echo $this->get_field_id( 'content' ); ?>" title="Enter hyperlink or message" name="<?php echo $this->get_field_name( 'content' ); ?>" type="text" value="<?php echo $content; ?>" <?php /* echo $permalink ? "disabled=\"disabled\"" : ""; */ ?> style="background:<?php echo $permalink ? "#F5F5F5" : "#FFFFFF"; ?>" />
		</p>
		
		<!-- Should use page permalink for content? -->
		<p>
			<input type="checkbox" class="checkbox" id="<?php echo $this->get_field_id( 'permalink' ); ?>" name="<?php echo $this->get_field_name('permalink'); ?>" title="Check to use blog URL as content" <?php checked($permalink); ?> onchange="permalink_change_<?php echo $safeid; ?>()" />
			<label for="<?php echo $this->get_field_id( 'permalink' ); ?>" title="Check to use blog URL as content"><?php _e( 'Use permalink as content' ); ?></label>
		</p>
		
		<!-- Module size "n" or image size "width x height" -->
		<p>
			<label for="<?php echo $this->get_field_id( 'module_size' ); ?>" title="Enter module size (number between 1 and 20) or image size (width x height where both dimensions are equal)"><?php _e( 'Size:' ); ?></label> 
			<input class="widefat" id="<?php echo $this->get_field_id( 'module_size' ); ?>" title="Enter module size (number between 1 and 20) or image size (width x height where both dimensions are equal)" name="<?php echo $this->get_field_name( 'module_size' ); ?>" type="text" value="<?php echo $module_size; ?>" />
		</p>
		
		<!-- Padding, number of modules -->
		<p>
			<label for="<?php echo $this->get_field_id( 'padding' ); ?>" title="Enter value between 0 and 4"><?php _e( 'Padding:' ); ?></label> 
			<input class="widefat" id="<?php echo $this->get_field_id( 'padding' ); ?>" title="Enter value between 0 and 4" name="<?php echo $this->get_field_name( 'padding' ); ?>" type="text" value="<?php echo $padding; ?>" />
		</p>
		
		<!-- Caption below QR Code -->
		<p class="advanced_<?php echo $safeid; ?>" style="display: <?php echo $advanced ? "block" : "none"; ?>">
			<label for="<?php echo $this->get_field_id( 'caption' ); ?>" title="Enter text to appear below the QR Code"><?php _e( 'Caption:' ); ?></label> 
			<input class="widefat" id="<?php echo $this->get_field_id( 'caption' ); ?>" title="Enter text to appear below the QR Code" name="<?php echo $this->get_field_name( 'caption' ); ?>" type="text" value="<?php echo $caption; ?>" />
		</p>
		
		<!-- Tooltip, "title" attribute for "img" -->
		<p class="advanced_<?php echo $safeid; ?>" style="display: <?php echo $advanced ? "block" : "none"; ?>">
			<label for="<?php echo $this->get_field_id( 'tooltip' ); ?>" title="Enter text to appear on mouse over ('title' attribute)"><?php _e( 'Tooltip:' ); ?></label> 
			<input class="widefat" id="<?php echo $this->get_field_id( 'tooltip' ); ?>" title="Enter text to appear on mouse over ('title' attribute)" name="<?php echo $this->get_field_name( 'tooltip' ); ?>" type="text" value="<?php echo $tooltip; ?>" />
		</p>
		
		<!-- img[alt], "alt" attribute for "img" -->
		<p class="advanced_<?php echo $safeid; ?>" style="display: <?php echo $advanced ? "block" : "none"; ?>">
			<label for="<?php echo $this->get_field_id( 'alt' ); ?>" title="Enter text for 'alt' attribute"><?php _e( '"alt" attribute:' ); ?></label> 
			<input class="widefat" id="<?php echo $this->get_field_id( 'alt' ); ?>" title="Enter text for 'alt' attribute" name="<?php echo $this->get_field_name( 'alt' ); ?>" type="text" value="<?php echo $alt; ?>" />
		</p>
		
		<!-- Module color: hex or color name -->
		<p class="advanced_<?php echo $safeid; ?>" style="display: <?php echo $advanced ? "block" : "none"; ?>">
			<label for="<?php echo $this->get_field_id( 'module_color' ); ?>" title="Enter color name or #[AA]RRGGBB hex value, e.g. 'black' or '#000000'"><?php _e( 'Module color:' ); ?></label> 
			<input class="widefat" id="<?php echo $this->get_field_id( 'module_color' ); ?>" title="Enter color name or #[AA]RRGGBB hex value, e.g. 'black' or '#000000'" name="<?php echo $this->get_field_name( 'module_color' ); ?>" type="text" value="<?php echo $module_color; ?>" />
		</p>
		
		<!-- Background color: hex or color name -->
		<p class="advanced_<?php echo $safeid; ?>" style="display: <?php echo $advanced ? "block" : "none"; ?>">
			<label for="<?php echo $this->get_field_id( 'background_color' ); ?>" title="Enter color name or #[AA]RRGGBB hex value, e.g. 'transparent' or '#00FFFFFF'"><?php _e( 'Background color:' ); ?></label> 
			<input class="widefat" id="<?php echo $this->get_field_id( 'background_color' ); ?>" title="Enter color name or #[AA]RRGGBB hex value, e.g. 'transparent' or '#00FFFFFF'" name="<?php echo $this->get_field_name( 'background_color' ); ?>" type="text" value="<?php echo $background_color; ?>" />
		</p>
		
		<!-- Image alignment: left, center, right -->
		<p class="advanced_<?php echo $safeid; ?>" style="display: <?php echo $advanced ? "block" : "none"; ?>">
			<label for="<?php echo $this->get_field_id( 'align' ); ?>"><?php _e( 'Alignment:' ); ?></label> 
			<select id="<?php echo $this->get_field_id( 'align' ); ?>" name="<?php echo $this->get_field_name( 'align' ); ?>" style="width:100%;">
				<?php
					$options = array('Left', 'Center', 'Right');
					for ($i = 1; $i <= count($options); $i++)
					{
						?><option value="<?php echo $i; ?>" <?php echo $align == "$i" ? 'selected="selected"' : ''; ?>><?php echo $options[$i - 1]; ?></option><?php
					}
				?>
			</select>
		</p>
		
		<!-- Show advanced features? -->
		<p>
			<input type="checkbox" class="checkbox" id="<?php echo $this->get_field_id( 'advanced' ); ?>" name="<?php echo $this->get_field_name( 'advanced' ); ?>" title="Check to show additional options" <?php checked($advanced); ?> <?php echo $update ? '' : 'disabled="disabled"' ?> onchange="advanced_change_<?php echo $safeid; ?>()" />
			<label for="<?php echo $this->get_field_id( 'advanced' ); ?>" title="Check to show additional options. Note: widget must be saved first to enable this feature."><?php _e( 'Show advanced options' ); ?></label>
		</p>
		
		<script type="text/javascript">
		function permalink_change_<?php echo $safeid; ?>()
		{
			var input = document.getElementById("<?php echo $this->get_field_id( 'content' ); ?>");
			var checkbox = document.getElementById("<?php echo $this->get_field_id( 'permalink' ); ?>");
			
			//input.disabled = checkbox.checked; //Make the input box inactive
			input.style.background = checkbox.checked ? "#F5F5F5" : "#FFFFFF"; //Keep the input active but mark it as unused
		}
		
		function advanced_change_<?php echo $safeid; ?>()
		{
			//Find hidden input for reference
			var hidden = document.getElementById("<?php echo $this->get_field_id( 'wid' ); ?>");
			var checkbox = document.getElementById("<?php echo $this->get_field_id( 'advanced' ); ?>");
			
			//Loop through all "p" elements
			var nodes = hidden.parentNode.childNodes;
			for (var i = 0; i < nodes.length; i++)
			{
				var element = nodes[i];
				if (element.className == "advanced_<?php echo $safeid; ?>")
				{
					//Toggle "p" container visible/hidden
					element.style.display = checkbox.checked ? "block" : "none";
				}
			}
		}
		</script>
		<?php
	}
}
?>