/**************************************************************************************//**
 * Generates a [qrcode] tag sample.
 ******************************************************************************************/
function inject_esponce_qrcode()
{
	//Get parameters
	var content = "Hello World";
	var size = 8;
	var padding = 4;
	var foreground = "#000000";
	var background = "#FFFFFF";
	
	//Example: [qrcode content="Hello World" size="8" padding="4" foreground="#000000" background="#FFFFFF"]
	
	//Build short code
	var shortcode = '[qrcode';
	shortcode += ' content="' + content + '"';
	if (size && size >= 1)
	{
		shortcode += ' size="' + size + '"';
	}
	if (padding && padding >= 0)
	{
		shortcode += ' padding="' + padding + '"';
	}
	if (foreground && (/(#[0-9A-F]{3})|(#[0-9A-F]{6})|([A-Z]{2})/i).test(foreground))
	{
		shortcode += ' foreground="' + foreground + '"';
	}
	if (background && (/(#[0-9A-F]{3})|(#[0-9A-F]{6})|([A-Z]{2})/i).test(background))
	{
		shortcode += ' background="' + background + '"';
	}
	shortcode += ']';
	return shortcode;
}

/**************************************************************************************//**
 * Trim left and right spaces.
 ******************************************************************************************/
String.prototype.esponce_trim = function()
{
	return this.replace(/^\s\s*/, "").replace(/\s\s*$/, "");
};

/**************************************************************************************//**
 * Encodes shortcode component value.
 ******************************************************************************************/
function esponce_encode_shortcode_value(value)
{
	//Value must be set to continue
	if (!value) return value;
	
	var result = value;
	result = result.replace(/"/g, '&quot;'); //Escape all double quotes
	result = result.replace(/(\r\n|\r|\n)/g, '&br;'); //New lines
	return result;
}

/**************************************************************************************//**
 * Decodes shortcode component value.
 ******************************************************************************************/
function esponce_decode_shortcode_value(value)
{
	//Value must be set to continue
	if (!value) return value;
	
	var result = value;
	result = result.replace(/&quot;/g, '"'); //Double quotes
	result = result.replace(/&br;/g, '\n'); //New lines
	return result;
}

/**************************************************************************************//**
 * Encodes HTML attribute.
 ******************************************************************************************/
function esponce_encode_html_attr(value)
{
	//Value must be set to continue
	if (!value) return value;
	
	var result = value;
	result = result.replace(/"/g, '&quot;'); //Double quotes
	return result;
}

/**************************************************************************************//**
 * Decodes HTML attribute.
 ******************************************************************************************/
function esponce_decode_html_attr(value)
{
	//Value must be set to continue
	if (!value) return value;
	
	var result = value;
	result = result.replace(/&quot;/g, '"'); //Double quotes
	return result;
}

/**************************************************************************************//**
 * Gets parameters from <img src="..." title="..." />
 ******************************************************************************************/
function esponce_get_url_parameters(url) //Could be 'url' or img element
{
	var m, im = url;
	var result =
	{
		//Get from URL
		content: (m = (/\?content=(.+?)[&"]/).exec(im)) ? decodeURIComponent("" + m[1]) : "",
		foreground: (m = (/&(amp;)?foreground=(.+?)[&"]/).exec(im)) ? decodeURIComponent("" + m[2]) : "",
		background: (m = (/&(amp;)?background=(.+?)[&"]/).exec(im)) ? decodeURIComponent("" + m[2]) : "",
		size: (m = (/&(amp;)?size=(\d+(\s*[Xx]\s*\d+)?)[&"]/).exec(im)) ? m[2].replace(" ", "") : "",
		padding: (m = (/&(amp;)?padding=(\d+?)[&"]/).exec(im)) ? m[2] : "",
		version: (m = (/&(amp;)?version=([\d\-]+?)[&"]/).exec(im)) ? m[2] : "",
		ec: (m = (/&(amp;)?ec=([LMHQ])[&"]/).exec(im)) ? m[2] : "",
		
		//Get from "img" element
		alt: (m = (/alt="(.*?)"/).exec(im)) ? esponce_decode_html_attr(m[1]) : "",
		tooltip: (m = (/title="(.*?)"/).exec(im)) ? esponce_decode_html_attr(m[1]) : "",
		style: (m = (/style="(.*?)"/).exec(im)) ? esponce_decode_html_attr(m[1]) : "",
		className: (m = (/class="(.*?)"/).exec(im)) ? m[1] : "",
		width: (m = (/width="(.*?)"/).exec(im)) ? m[1] : "",
		height: (m = (/height="(.*?)"/).exec(im)) ? m[1] : ""
	};
	return result;
}

/**************************************************************************************//**
 * Gets parameters from <img src="..." title="..." />
 ******************************************************************************************/
function esponce_get_shortcode_parameters(shortcode)
{
	var m, im = shortcode;
	var result =
	{
		content: (m = (/content="(.*?)"/).exec(im)) ? esponce_decode_shortcode_value("" + m[1]) : "",
		caption: (m = (/caption="(.*?)"/).exec(im)) ? esponce_decode_shortcode_value("" + m[1]) : "",
		tooltip: (m = (/tooltip="(.*?)"/).exec(im)) ? esponce_decode_shortcode_value("" + m[1]) : "",
		alt: (m = (/alt="(.*?)"/).exec(im)) ? esponce_decode_shortcode_value("" + m[1]) : "",
		foreground: (m = (/foreground="(.*?)"/).exec(im)) ? m[1] : "",
		background: (m = (/background="(.*?)"/).exec(im)) ? m[1] : "",
		size: (m = (/size="(.*?)"/).exec(im)) ? m[1] : "",
		padding: (m = (/padding="(.*?)"/).exec(im)) ? m[1] : "",
		version: (m = (/version="(.*?)"/).exec(im)) ? m[1] : "",
		em: (m = (/em="(.*?)"/).exec(im)) ? m[1] : "",
		ec: (m = (/ec="(.*?)"/).exec(im)) ? m[1] : "",
		className: (m = (/class="(.*?)"/).exec(im)) ? m[1] : "",
		style: (m = (/style="(.*?)"/).exec(im)) ? m[1] : "",
		width: (m = (/width="(.*?)"/).exec(im)) ? m[1] : "",
		height: (m = (/height="(.*?)"/).exec(im)) ? m[1] : ""
	};
	return result;
}

/**************************************************************************************//**
 * Gets a built URL from parameters
 * @param p Parameters, e.g. { content: "Hello", size = 8 }
 ******************************************************************************************/
function esponce_build_url(p)
{
	var url = "http://www.esponce.com/api/v3/generate?content=" + encodeURIComponent(p.content) + "&format=png";
	url += (p.foreground && p.foreground.length > 0 ? "&foreground=" + encodeURIComponent(p.foreground) : "");
	url += (p.background && p.background.length > 0 ? "&background=" + encodeURIComponent(p.background) : "");
	url += (p.size && p.size.length > 0 && parseInt(p.size, 10) > 0 ? "&size=" + p.size : "");
	url += (p.padding && p.padding.length > 0 && parseInt(p.padding, 10) >= 0 ? "&padding=" + p.padding : "");
	url += (p.version && p.version.length > 0 && parseInt(p.version, 10) > 0 ? "&version=" + p.version : "");
	url += (p.em && p.em.length > 0 ? "&em=" + p.em : "");
	url += (p.ec && p.ec.length > 0 ? "&ec=" + p.ec : "");
	return url;
}

/**************************************************************************************//**
 * Gets a built shortcode from parameters
 * @param p Parameters, e.g. { content: "Hello", size = 8 }
 * @example [qrcode content="Hello World" size="8" padding="4" foreground="#000000" background="#FFFFFF"]
 ******************************************************************************************/
function esponce_build_shortcode(p)
{
	var code = '[qrcode content="' + esponce_encode_shortcode_value(p.content) + '"';
	code += (p.caption && p.caption.length > 0 ? ' caption="' + esponce_encode_shortcode_value(p.caption) + '"' : "");
	code += (p.tooltip && p.tooltip.length > 0 ? ' tooltip="' + esponce_encode_shortcode_value(p.tooltip) + '"' : "");
	code += (p.alt && p.alt.length > 0 ? ' alt="' + esponce_encode_shortcode_value(p.alt) + '"' : "");
	code += (p.foreground && p.foreground.length > 0 ? ' foreground="' + p.foreground + '"' : "");
	code += (p.background && p.background.length > 0 ? ' background="' + p.background + '"' : "");
	code += (p.size && p.size.length > 0 /* && parseInt(size, 10) > 0 */ ? ' size="' + p.size + '"' : "");
	code += (p.padding && p.padding.length > 0 && parseInt(p.padding, 10) >= 0 ? ' padding="' + p.padding + '"' : "");
	code += (p.version && p.version.length > 0 && parseInt(p.version, 10) > 0 ? ' version="' + p.version + '"' : "");
	code += (p.em && p.em.length > 0 ? ' em="' + p.em + '"' : "");
	code += (p.ec && p.ec.length > 0 ? ' ec="' + p.ec + '"' : "");
	
	var className = p.className;
	if (className && className.length > 0)
	{
		//Remove repeating "mceEsponceQRCode" class
		className = className.replace(/mceEsponceQRCode/g, "");
		className = className.esponce_trim();
		code += (className.length > 0 ? ' class="' + className + '"' : "");
	}
	
	code += (p.style && p.style.length > 0 ? ' style="' + p.style + '"' : "");
	code += (p.width && p.width.length > 0 ? ' width="' + p.width + '"' : "");
	code += (p.height && p.height.length > 0 ? ' height="' + p.height + '"' : "");
	code += ']';
	return code;
}

/**************************************************************************************//**
 * Initialize TinyMCE features
 ******************************************************************************************/
(function ()
{
	var enableKeyboardShortcut = false;
	
	/**************************************************************************************//**
	 * Register button in the TinyMCE editor.
	 ******************************************************************************************/
	tinymce.create('tinymce.plugins.inject_esponce_qrcode',
	{
		init: function (editor, url)
		{
			//Register command to open dialog
			editor.addCommand("WP_Esponce_QRCode", function ()
			{
				var element, query = "";
				
				//Get passed parameters - bad work-around but this is the only way to get parameters
				var scripts = document.getElementsByTagName("script");
				for (var i = 0; i < scripts.length; i++)
				{
					var script = scripts[i];
					if (script.src && script.src.match(/plugin\.js\?su=/gi))
					{
						var value = (/plugin\.js\?su=(.*?)&(amp;)?au=(.*?)[&"]/gi).exec(script.src);
						query = "?su=" + value[1]; //site_url
						query += "&au=" + value[3]; //admin_url
						break;
					}
					else if (script.innerHTML && script.innerHTML.match(/plugin\.js\?su=/gi))
					{
						var value = (/plugin\.js\?su=(.*?)&au=(.*?)[&"]/gi).exec(script.innerHTML);
						query = "?su=" + value[1]; //site_url
						query += "&au=" + value[2]; //admin_url
						break;
					}
				}		
			
				//Directly insert QR Code syntax, i.e. insert without opening popup window
				//editor.execCommand('mceInsertContent', false, inject_esponce_qrcode());
				
				//Check if the selected element is QR Code image
				if (editor.selection && (element = editor.selection.getNode()) && element.nodeName.toLowerCase() == "img" && element.className && element.className.indexOf("mceEsponceQRCode") >= 0)
				{
					//Get parameters
					var p = esponce_get_url_parameters(element.src);
					
					//Build query string
					query += "&content=" + encodeURIComponent(p.content);
					query += "&foreground=" + encodeURIComponent(p.foreground);
					query += "&background=" + encodeURIComponent(p.background);
					query += "&size=" + p.size;
					query += "&padding=" + p.padding;
					query += "&version=" + p.version;
					query += "&ec=" + p.ec;
					
					query += (element.width ? "&width=" + element.width : "");
					query += (element.width ? "&height=" + element.height : "");
					query += (element.title ? "&tooltip=" + encodeURIComponent(element.title) : "");
					query += (element.alt ? "&alt=" + encodeURIComponent(element.alt) : "");
					query += (element.style.cssText ? "&style=" + encodeURIComponent(element.style.cssText) : "");
					query += (element.className ? "&class=" + encodeURIComponent(element.className) : "");
				}
				
				//Open popup window
				editor.windowManager.open(
				{
					inline: 1,
					width: 480,
					height: 330, //330 means 280px for dialog
					url: url + "/dialog.php" + query,
					title: "Esponce QR Code Generator"
				},
				//Custom parameters
				{
					mce: tinyMCE,
					editor: editor
				});
			});
			
			//Register button to trigger command
			editor.addButton('inject_esponce_qrcode',
			{
				title: 'Insert QR Code image' + (enableKeyboardShortcut ? " (Alt + Shift + Q)" : ""),
				image: url + "/icon.png",
				cmd: "WP_Esponce_QRCode"
			});
			
			//Register keyboard shortcut
			if (enableKeyboardShortcut)
			{
				//NOTE: "alt+shift+q" may already be defined for "Quote" command
				editor.addShortcut("alt+shift+q", "Esponce QR Code Generator", "WP_Esponce_QRCode");
			}
			
			//Add listener to handle preview/short code display
			this._handleQRCode(editor, url);
		},
		getInfo: function ()
		{
			var info = {
				longname: "Esponce QR Code Generator Dialog",
				author: "Esponce",
				authorurl: "http://www.esponce.com",
				infourl: "",
				version: "1.4"
			}
			return info;
		},
		_handleQRCode: function (editor, url)
		{
			//Display "qrcode" instead if "img" in element path
			editor.onPostRender.add(function()
			{
				if (editor.theme.onResolveName)
				{
					editor.theme.onResolveName.add(function(th, o)
					{
						if (o.node.nodeName == 'IMG' && editor.dom.hasClass(o.node, 'mceEsponceQRCode'))
						{
							//Override name in element path bar
							o.name = 'qrcode';
						}
					});
				}
			});

			//Replace short code with image (Visual mode)
			editor.onBeforeSetContent.add(function(editor, o)
			{
				if (o.content)
				{
					var match;
					
					//Find "qrcode" short codes
					while (match = (/\[qrcode(.*?)\]/gi).exec(o.content))
					{
						var shortcode = "[qrcode" + match[1] + "]";
						
						//Parse short code to get parameters
						var p = esponce_get_shortcode_parameters(shortcode);
						p.content = p.content.replace("\\\"", "\""); //Fix escaped double quotes
						var caption = p.caption ? p.caption.replace("\"", "'") : ""; //Escape double quotes
						var tooltip = p.tooltip ? p.tooltip.replace("\"", "'") : ""; //Escape double quotes
						var alt = p.alt ? p.alt.replace("\"", "'") : ""; //Escape double quotes
						var style = p.style ? p.style.replace("\"", "'") : ""; //Escape double quotes
						var className = p.className ? p.className.replace("\"", "'") : ""; //Escape double quotes
						
						//Build URL to generate QR Code image
						var url = esponce_build_url(p) + "&source=wp";
						
						//Build "img" tag to display preview
						var code = '<img src="' + url + '"';
						code += (p.alt && p.alt.length > 0 ? ' alt="' + esponce_encode_html_attr(p.alt) + '"' : "");
						code += (p.tooltip && p.tooltip.length > 0 ? ' title="' + esponce_encode_html_attr(p.tooltip) + '"' : "");
						code += (p.width && p.width.length > 0 ? ' width="' + p.width + '"' : "");
						code += (p.height && p.height.length > 0 ? ' height="' + p.height + '"' : "");
						code += ' class="mceEsponceQRCode' + (p.className && p.className.length > 0 ? " " + p.className : "") + '"';
						code += (p.style && p.style.length > 0 ? ' style="' + esponce_encode_html_attr(p.style) + '"' : "");
						code += ' />';
						
						//Build caption part
						//if (caption && caption.length > 0)
						//{
						//  code += '<span class="mceEsponceQRCodeCaption">' + caption + '</span>';
						//}
						
						//Replace short code with HTML element
						o.content = o.content.replace(shortcode, code);
					}
				}
			});

			//Replace image with short code (HTML mode)
			editor.onPostProcess.add(function(editor, o)
			{
				if (o.get)
				{
					o.content = o.content.replace(/<img[^>]+>/g, function(im)
					{
						if (im.match(/class="[\w\d\s\-_]*\s*mceEsponceQRCode[\w\d\s\-_]*\s*"/))
						{
							//Parse "img" to get parameters
							var p = esponce_get_url_parameters(im);
							
							//Build short code from parameters
							var code = esponce_build_shortcode(p);
							im = code;
						}
						return im;
					});
				}
			});

			//Set active button if user selected "qrcode"
			editor.onNodeChange.add(function(editor, cm, n)
			{
				cm.setActive('inject_esponce_qrcode', n.nodeName === 'IMG' && editor.dom.hasClass(n, 'mceEsponceQRCode'));
			});
		}
	});

	tinymce.PluginManager.add('inject_esponce_qrcode', tinymce.plugins.inject_esponce_qrcode);
})();